SET SERVEROUTPUT ON

--
-- Drop Table procedure
CREATE OR REPLACE PROCEDURE drop_if_exists(table_name VARCHAR2)
IS
  table_inexistent EXCEPTION;                  -- table does not exists
  PRAGMA EXCEPTION_INIT(table_inexistent,-942);-- associate with error ORA-00942
BEGIN 
  EXECUTE IMMEDIATE ('DROP TABLE '|| table_name || ' CASCADE CONSTRAINTS');
EXCEPTION
WHEN table_inexistent THEN
  RETURN;
END drop_if_exists;
/

--
-- Drop column procedure
CREATE OR REPLACE PROCEDURE drop_column_if_exists(table_name  VARCHAR2
                                                 ,column_name VARCHAR2)
IS
  column_inexistent EXCEPTION;                  -- column does not exist
  PRAGMA EXCEPTION_INIT(column_inexistent,-904);-- associate with error ORA-00904
BEGIN
  EXECUTE IMMEDIATE ('ALTER TABLE '|| table_name || ' DROP (' || column_name || ')');
EXCEPTION
WHEN column_inexistent THEN
  RETURN;
END drop_column_if_exists;
/

--
-- Rename column procedure
CREATE OR REPLACE PROCEDURE rename_column(table_name  VARCHAR2
                                         ,column_name VARCHAR2
                                         ,column_type VARCHAR2)
IS
BEGIN
  drop_column_if_exists(table_name, column_name || '_tmp');
  EXECUTE IMMEDIATE ('ALTER TABLE ' || table_name || ' ADD (' || column_name || '_tmp ' || column_type || ')');
  EXECUTE IMMEDIATE ('UPDATE ' || table_name || ' SET ' || column_name || '_tmp=' || column_name);
  EXECUTE IMMEDIATE ('COMMIT');
  EXECUTE IMMEDIATE ('ALTER TABLE ' || table_name || ' DROP (' || column_name || ')');
  EXECUTE IMMEDIATE ('ALTER TABLE ' || table_name || ' RENAME COLUMN ' || column_name || '_tmp TO ' || column_name);
END rename_column;
/

--
-- Main
DECLARE
  v_major INTEGER := 0;
  v_minor INTEGER := 0;
  v_patch INTEGER := 0;
  
  v_req_major     CONSTANT INTEGER  := 2;
  v_req_minor     CONSTANT INTEGER  := 1;
  v_req_patch_min CONSTANT INTEGER  := 0;
  v_req_patch_max CONSTANT INTEGER  := 0;
  
  v_next_major CONSTANT INTEGER := 2;
  v_next_minor CONSTANT INTEGER := 2;
  v_next_patch CONSTANT INTEGER := 0;
 
BEGIN

  DBMS_OUTPUT.PUT_LINE('Applying Patch...');
  -- Get schema version
  SELECT MAJOR,MINOR,PATCH INTO v_major,v_minor,v_patch FROM T_SCHEMA_VERS WHERE ROWNUM <=1;
  -- check version
  IF v_major = v_req_major AND  v_minor = v_req_minor AND (v_patch BETWEEN  v_req_patch_min AND v_req_patch_max) THEN
     -- apply patch

     -- Add submit_host and max_time_in_queue to T_JOB table
     EXECUTE IMMEDIATE ('ALTER TABLE T_JOB ADD ( SUBMIT_HOST VARCHAR2(100) )');
     EXECUTE IMMEDIATE ('ALTER TABLE T_JOB ADD ( MAX_TIME_IN_QUEUE INTEGER )');
     
     -- Add current_failures and finish_time to T_FILE
     EXECUTE IMMEDIATE ('ALTER TABLE T_FILE ADD ( CURRENT_FAILURES INTEGER )');
     EXECUTE IMMEDIATE ('ALTER TABLE T_FILE ADD ( FINISH_TIME TIMESTAMP WITH TIME ZONE )');
   
     -- Add estimated_wait_time to T_TRANSFER_PERIOD_STATS
     EXECUTE IMMEDIATE ('ALTER TABLE T_TRANSFER_PERIOD_STATS ADD ( ESTIMATED_WAIT_TIME INTEGER )');

     -- Add T_CREDENTIAL table
     EXECUTE IMMEDIATE('CREATE TABLE t_credential (
        dlg_id        VARCHAR2(100)
        ,dn           VARCHAR2(255)
        ,certificate  CLOB
                      CONSTRAINT cred_certificate_not_null NOT NULL
        ,priv_key     CLOB
                      CONSTRAINT cred_priv_key_not_null NOT NULL
        ,voms_attrs   CLOB
        ,CONSTRAINT credential_pk PRIMARY KEY (dlg_id, dn)
	)');
    
     -- Add T_CREDENTIAL_CACHE table
     EXECUTE IMMEDIATE('CREATE TABLE t_credential_cache (
        dlg_id        VARCHAR2(100)
        ,dn           VARCHAR2(255)
        ,cert_request CLOB
                      CONSTRAINT cred_cache_cert_req_not_null NOT NULL
        ,priv_key     CLOB
                      CONSTRAINT cred_cache_priv_key_not_null NOT NULL
        ,voms_attrs   CLOB
        ,CONSTRAINT credential_cache_pk PRIMARY KEY (dlg_id, dn)
        )');

     -- Update Schema Version
     UPDATE T_SCHEMA_VERS SET MAJOR=v_next_major, MINOR=v_next_minor, PATCH=v_next_patch; 
     DBMS_OUTPUT.PUT_LINE('SUCCESS: Patch applied. New version is ' || TO_CHAR(v_next_major) || '.' || TO_CHAR(v_next_minor) || '.' || TO_CHAR(v_next_patch) );
  ELSIF v_major = v_next_major AND  v_minor = v_next_minor AND v_patch = v_next_patch THEN
     DBMS_OUTPUT.PUT_LINE('SUCCESS: Patch already applied. Version is ' || TO_CHAR(v_next_major) || '.' || TO_CHAR(v_next_minor) || '.' || TO_CHAR(v_next_patch) );
  ELSE
     DBMS_OUTPUT.PUT_LINE('ERROR: Cannot apply patch. Version ' || TO_CHAR(v_major) || '.' || TO_CHAR(v_minor) || '.' || TO_CHAR(v_patch) || ' does not match the required one (' || TO_CHAR(v_req_major) || '.' || TO_CHAR(v_req_minor) || '.' || TO_CHAR(v_req_patch_min) || '-' ||  TO_CHAR(v_req_patch_max) || ')');
  END IF;
END;
/
DROP PROCEDURE drop_if_exists;
DROP PROCEDURE drop_column_if_exists;
DROP PROCEDURE rename_column;

