SET SERVEROUTPUT ON

--
-- Main
DECLARE
  v_major INTEGER := 0;
  v_minor INTEGER := 0;
  v_patch INTEGER := 0;
  
  v_req_major     CONSTANT INTEGER  := 2;
  v_req_minor     CONSTANT INTEGER  := 2;
  v_req_patch_min CONSTANT INTEGER  := 0;
  v_req_patch_max CONSTANT INTEGER  := 0;
  
  v_next_major CONSTANT INTEGER := 2;
  v_next_minor CONSTANT INTEGER := 2;
  v_next_patch CONSTANT INTEGER := 1;

--
-- Add index if it does not exist
PROCEDURE add_index(index_name  VARCHAR2
                   ,index_stmt  VARCHAR2)
IS
  index_exists EXCEPTION;                  --index already exists
  PRAGMA EXCEPTION_INIT(index_exists,-955);-- associate with error ORA-00955
BEGIN
  EXECUTE IMMEDIATE ('CREATE INDEX ' || index_name || ' ON ' || index_stmt || ' ONLINE');
  DBMS_OUTPUT.PUT_LINE('INFO: Index [' || index_name || '] added');
EXCEPTION
WHEN index_exists THEN
  DBMS_OUTPUT.PUT_LINE('INFO: Index [' || index_name || '] already exists. Skip it');
  RETURN;
END; --add_index
 
BEGIN

  DBMS_OUTPUT.PUT_LINE('Applying Patch...');
  -- Get schema version
  SELECT MAJOR,MINOR,PATCH INTO v_major,v_minor,v_patch FROM T_SCHEMA_VERS WHERE ROWNUM <=1;
  -- check version
  IF v_major = v_req_major AND  v_minor = v_req_minor AND (v_patch BETWEEN  v_req_patch_min AND v_req_patch_max) THEN
     -- apply patch

     --------------------------------------------------------------------------------------
     -- Index suggested by M. Anjo in order to improve performances and reduce the DB load
     --------------------------------------------------------------------------------------
     add_index('transfer_key_select','t_transfer(transfer_state,file_id,request_id,transfer_id,transfer_time)');
     add_index('idx_report_file'    ,'t_file(finish_time,file_state,file_id,job_id,filesize)');
     add_index('idx_report_transfer','t_transfer(transfer_state,reason_class,file_id,duration,bytes_written)');
     add_index('idx_report_job'     ,'t_job(channel_name,vo_name,job_id)');
     -- commit changes
     EXECUTE IMMEDIATE ('COMMIT');

     -- Update Schema Version
     UPDATE T_SCHEMA_VERS SET MAJOR=v_next_major, MINOR=v_next_minor, PATCH=v_next_patch; 
     DBMS_OUTPUT.PUT_LINE('SUCCESS: Patch applied. New version is ' || TO_CHAR(v_next_major) || '.' || TO_CHAR(v_next_minor) || '.' || TO_CHAR(v_next_patch) );
  ELSIF v_major = v_next_major AND  v_minor = v_next_minor AND v_patch = v_next_patch THEN
     DBMS_OUTPUT.PUT_LINE('SUCCESS: Patch already applied. Version is ' || TO_CHAR(v_next_major) || '.' || TO_CHAR(v_next_minor) || '.' || TO_CHAR(v_next_patch) );
  ELSE
     DBMS_OUTPUT.PUT_LINE('ERROR: Cannot apply patch. Version ' || TO_CHAR(v_major) || '.' || TO_CHAR(v_minor) || '.' || TO_CHAR(v_patch) || ' does not match the required one (' || TO_CHAR(v_req_major) || '.' || TO_CHAR(v_req_minor) || '.' || TO_CHAR(v_req_patch_min) || '-' ||  TO_CHAR(v_req_patch_max) || ')');
  END IF;
END;
/
